"""
This module handles the 'plop' command when used to copy an already
existing file (in the package) to the user's cwd.
"""
import os
import shutil

from geninstaller.helpers import set_executable

BASE_DIR = os.path.abspath(os.path.dirname(__file__))


def plop_installer():
    "plop's the installer in your cwd"
    file = os.path.join(BASE_DIR, "plop/installer/installer")
    cwd = os.getcwd()
    shutil.copy(file, cwd)
    set_executable(os.path.join(cwd, 'installer'))
    print(f"installer plopped in {cwd}")



#! /usr/bin/env python3
# coding: utf-8
import os

from flamewok.cli import cli
from flamewok import color as c
from silly_db.db import DB

from geninstaller import __version__
from geninstaller import core
from geninstaller import ploppers
from geninstaller.helpers import (
    GI_DIR,
    APP_FILES_DIR,
    DB_FILE,
    no_db,
    display_list,
)


def list(*args):
    if no_db():
        return
    db = DB(
        file=DB_FILE,
        base=GI_DIR,
        # migrations_dir="None",  # delete with silly-db v 1.1.4
    )
    App = db.model("application")
    apps = App.all()
    display_list(apps)


def open_geninstaller_dir(*args):
    """Directory where the database is installed localy"""
    if no_db():
        return
    os.system(f"xdg-open {GI_DIR}")


def open_apps_dir(*args):
    if no_db():
        return
    os.system(f"xdg-open {APP_FILES_DIR}")


def search(name=None, *args):
    name = name
    if name is None:
        return list()
    if no_db():
        return
    db = DB(
        file=DB_FILE,
        base=GI_DIR,
    )
    apps = db.select(f"* FROM application WHERE name LIKE '%{name}%'")
    display_list(apps)


def cmd():
    routes = [
        "HELP",
        ("-h", cli.help, "display this help"),
        ("--help", cli.help, "idem"),
        ("", cli.help, "idem"),
        "ACTIONS",
        ('list', list, "list the applications installed with geninstaller"),
        ('search <name>', search,
            "search an application with an approximate name"),
        ('uninstall <name>', core.uninstall, (
            "uninstall an application with its exact name, "
            "use '' if the 'app name' contains a blank space")),
        "OPEN DIRECTORIES",
        ('open', open_apps_dir,
            "open the applications installation directory"),
        ('open database', open_geninstaller_dir,
            "open the geninstaller's database directory"),
        "FOR DEVELOPPERS",
        ('plop installer', ploppers.plop_installer,
            "provides a ready-to-complete-and-use "
            "'installer' template into your current working directory"),
        "_"*78,
        "ABOUT",
        "program: geninstaller",
        f"version: {__version__}",
        "home page : https://github.com/byoso/geninstaller",

    ]
    cli.route(*routes)


if __name__ == "__main__":
    cmd()


#! /usr/bin/env python3
# coding: utf-8

"""The core of geninstaller consist mostly in install and uninstall
applications"""

import os

from flamewok import color as c

from geninstaller.helpers import (
    APP_FILES_DIR,
    APP_DIR,
    get_db,
    clean_dir_name,
    create_desktop,
    create_dir,
    valid_for_installation,
)


def install(data):
    """Prepares the data before finalization"""
    # first, some data check
    valid_for_installation(data)

    # transforming datas
    categories = ""
    for category in data['categories']:
        categories += category + "/"
    # directory name:
    applications_files = APP_FILES_DIR + clean_dir_name(data['name'])
    # desktop file name:
    applications = APP_DIR + clean_dir_name(data['name']) + ".desktop"
    if data['terminal']:
        terminal = "true"
    else:
        terminal = "false"

    db_datas = {
        'name': data['name'].strip(),
        'exec': data['exec'],
        'comment': data['comment'],
        'terminal': terminal,
        'icon': data['icon'],
        'categories': categories,
        'applications_files': applications_files,
        'applications': applications,

    }
    all_datas = {
        'base_dir': data['base_dir'],
        **db_datas
    }
    # finallization:
    gi_db = get_db()
    App = gi_db.model("application")
    App.insert(**db_datas)
    create_dir(all_datas)
    create_desktop(all_datas)

    print(
        f"{c.success}geninstaller has successfuly installed "
        f"'{data['name']}' on your system{c.end}")
    print("please read the geninstaller's help to know how to use it:")
    print("$ geninstaller -h")


def uninstall(name, *args):
    if len(args) > 0:
        print(
            f"{c.warning}To many arguments given{c.end}\n"
            "If the name of your app contains multiple words, \n"
            "write it with quotes: 'your app name'"
            )
        return
    gi_db = get_db()
    App = gi_db.model("application")
    apps = App.filter(f"name='{name}'")
    if len(apps) < 1:
        print(f"'{name}' is not a geninstaller application")
        return
    app = App.get_id(apps[0].id)

    os.system(f"rm {app.applications}")
    os.system(f"rm -rf {app.applications_files}")
    App.delete(f"id={app.id}")
    print(
        f"{c.success}'{name}' has been successfuly "
        f"removed from your system{c.end}")




"""The helpers are mainly functions shared with the other parts of
geninstaller"""


import os
import stat
import shutil
from distutils.dir_util import copy_tree

from silly_db.db import DB
from flamewok import color as c

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

GI_DIR = os.path.expanduser(
    "~/.local/share/applications-files/.geninstaller/")
APP_FILES_DIR = os.path.expanduser(
    "~/.local/share/applications-files/")
APP_DIR = os.path.expanduser(
    "~/.local/share/applications/")
DB_FILE = os.path.expanduser(
    "~/.local/share/applications-files/.geninstaller/gi_db.sqlite3")


def abort(content):
    message = f"{c.warning}Aborted: {content}{c.end}"
    print(message)
    exit()


def no_forbidden(el):
    if ";" in el:
        abort(f"forbidden use of ';' in: '{el}'")


def autoinstall():
    """install the empty database"""
    if not os.path.exists(DB_FILE):
        copy_tree(
            BASE_DIR+'/plop/database', GI_DIR)
        print("geninstaller database initialized")


def set_executable(file) -> None:
    """set a file executable"""
    st = os.stat(file)
    os.chmod(file, st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)


def no_db():
    """check if the database already exists or not"""
    if not os.path.exists(DB_FILE):
        print("geninstaller's database has not been initialized.")
        return True
    else:
        return False


def get_db():
    if no_db:
        autoinstall()
    gi_db = DB(
            file=DB_FILE,
            base=GI_DIR,
        )
    return gi_db


def display_list(apps):
    """apps are a silly-db Selection"""
    print("="*80)
    print(f"{'Geninstaller: Installed Applications':^80}")
    print("="*80 + "|")
    if len(apps) == 0:
        print("\nNo geninstaller application found")
        return
    for app in apps:
        print(
            f"NAME: '{app.name}'\n"
            f"COMMENTS: {app.comment}\n"
            f"TERMINAL ?: {app.terminal}\n"
            f"CATEGORIES: {app.categories}"
            )
        print("_"*80 + "|")


def clean_dir_name(name):
    """Cleans up the name for the directory"""
    cleaner = name.strip()
    cleaned_name = ""
    for letter in cleaner:
        if letter in list(" ;,/\\"):
            cleaned_name += "_"
        else:
            cleaned_name += letter
    return cleaned_name


def valid_for_installation(data):
    """Check the datas before copying anything"""
    for el in data.values():
        if type(el) == str:
            no_forbidden(el)
    gi_db = get_db()
    App = gi_db.model("application")
    app = App.filter(f"name='{data['name']}'")
    if ("_" in data['name'] or "/" in data['name']
            or data['name'].startswith(".")):
        abort(
            "An app name must NOT contain '_' and '/', "
            "and must not begin with a '.'")
    if data['name'].lower().strip() in ["geninstaller", "geninstaller-gui"]:
        abort(f"'{data['name']}' is a reserved name")

    if type(data['terminal']) != bool:
        abort("The 'TERMINAL' value must be a boolean")

    for category in data['categories']:
        no_forbidden(category)
    if len(app) > 0:
        abort(
            f"An application called '{data['name']}'"
            " is already installed, change the current application's"
            f" name, or uninstall the other application first."
            )
    # check relative paths
    base_dir = data['base_dir']
    exec = os.path.join(base_dir, data['exec'])
    icon = os.path.join(base_dir, data['icon'])
    if not os.path.exists(exec):
        abort("Wrong path to exec")
    if data['icon'] != '' and not os.path.exists(icon):
        abort("Wrong path to icon")


def create_desktop(datas):
    """Create the .desktop file and copy it to ~/.local/share/applications"""
    file_name = datas['applications']
    destination_dir = datas['applications_files']
    name = datas['name']
    exec = os.path.join(destination_dir, datas['exec'])
    icon = os.path.join(destination_dir, datas['icon'])
    comment = datas['comment']
    terminal = datas['terminal']
    categories = datas['categories']
    desktop_categories = categories.replace("/", ";")
    content = (
        "[Desktop Entry]\n"
        f"Name={name}\n"
        f"Icon={icon}\n"
        f"Comment={comment}\n"
        f"Exec={exec}\n"
        f"Terminal={terminal}\n"
        f"Type=Application\n"
        f"Categories={desktop_categories}\n"
        )
    with open(file_name, "w") as file:
        file.write(content)
    set_executable(file_name)


def create_dir(datas):
    """Copy all the files in the root directory of the app to its
    right place, and ensure that the exec file is set 'executable'"""
    base_dir = datas['base_dir']
    destination_dir = datas['applications_files']
    shutil.copytree(
        base_dir, destination_dir)
    exec = os.path.join(destination_dir, datas['exec'])
    set_executable(exec)
